
#ifndef __MAXMIN_V2__
#define __MAXMIN_V2__
/** MaxMin utilities.
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (Wed, 03 Jul 2013) $
 * $Id: maxmin_v2.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#include <iostream>
#include <vector> 
using namespace std;

template <class T>
class Maxmin_db {
  const T *darr;
  T  dmax, dmin, a_mean;
  //  const float **farr2, *farr;
  //  const vector <T> *vfarr;
  float snr;
  long ndat;  /*Primary number of data */
  long ndat2;        /*data number for 2D array ndat*ndat2*/
  long imax;
  long imin;  /* primary index of 1D array*/
  long jmax;
  long jmin;  /* index for 2D array */
  long ixmax; /*used when index i is scaned up to ixmax instead of ndat*/
 private:
  void Warn(const char* str);

 public:
  Maxmin_db(void):ndat(0),ndat2(0),imax(0),imin(0),
		  jmax(0),jmin(0),ixmax(-1) {;};
   void Max_min2( const vector <T> arr[], const long n);
   void Max_min2( const T* arr, const long nx, const long ny);
   void Max_min1( const vector <T> &  arr);
   void Max_min1(const T*  arr, const long n);
   T Max(void){return(dmax);}
   T Min(void){return(dmin);}
   long  Imin(void){return(imin);}
   long  Imax(void){return(imax);}
   long  Jmin(void){return(jmin);}
   long  Jmax(void){return(jmax);}
   float Snr(void){return(snr);}
   T Mean(void){return(a_mean);}
};

inline double min_dbl(double a, double b){ return(a < b ? a : b);}
inline long  min_int(long a, long  b){ return(a < b ? a : b);}


template <class T>
void Maxmin_db<T>::Warn(const char *str)
{
  if(strlen(str) >0){
    cerr <<  "Error: array in pointer is null (" 
	 << str << ")" << endl;
    exit(EXIT_FAILURE);
  }else{
    cerr << "Error: array in pointer is null." << endl;
    exit(EXIT_FAILURE);
  }
}

//****** get MAX and MIN in a 2D double precision array ***************
template <class T>
void Maxmin_db<T>::Max_min2( const vector <T> arr[], const long n)
{
  int	i,j, ixmax, iymax, ixmin, iymin;

  ndat2 = n;

  ixmax = ixmin = iymin = iymax = 0;
  //  if((vfarr = arr) ==NULL) Warn("maxmin2fb");
  
  i=0;
  while(arr[i].size() ==0) i++;
  dmax = dmin = arr[i][0];    
  
  double  mf=0.;

  long icnt=0;
  for(i=0; i<ndat2; i++){
    ndat = arr[i].size();    
    for(j=0; j<ndat; j++){
      if( dmax < arr[i][j]){ 
	dmax = arr[i][j]; 
	ixmax = j;
	iymax = i;
      }
      if( dmin > arr[i][j]){ 
	dmin = arr[i][j];
	  ixmin = j;
	  iymin = i;
      }
      mf += arr[i][j];
      icnt++;
    }
  }
  mf -= dmax;
  a_mean = mf / (double)icnt;
  imax = ixmax;
  imin = ixmin;
  jmax = iymax;
  jmin = iymin;
  snr =  dmax/a_mean;
};

//****** get MAX and MIN in a 2D double precision array ***************
template <class T>
void Maxmin_db<T>::Max_min2( const T* arr, const long nx, const long ny)
{
  int	ixmax, iymax, ixmin, iymin;

  ndat2 = ny;
  ndat = nx;
  
  ixmax = ixmin = iymin = iymax = 0;
  dmax = dmin = arr[0];    
  
  T  mf=0.;

  long icnt=0;
  for(int i=0; i<ndat2; i++){
    for(int j=0; j<ndat; j++){
      if( dmax < arr[i*ndat+j]){ 
	dmax = arr[i*ndat+j]; 
	ixmax = j;
	iymax = i;
      }
      if( dmin > arr[i*ndat+j]){ 
	dmin = arr[i*ndat+j];
	  ixmin = j;
	  iymin = i;
      }
      mf += arr[i*ndat+j];
      icnt++;
    }
  }
  mf -= dmax;
  a_mean = mf / (double)icnt;
  imax = ixmax;
  imin = ixmin;
  jmax = iymax;
  jmin = iymin;
  snr =  dmax/a_mean;
};



/** get MAX and MIN in a 1D array *************/
template <class T>
void Maxmin_db <T> ::Max_min1( vector <T> const & arr){
  ndat = arr.size();
  
  if(ndat <1 ) Warn("max_min1");
  imax = imin =0;
  dmax = dmin = arr[0];
  
  double mf=0;    
  for(long i=0; i<ndat; i++){
    if( dmax < (arr)[i]){
      dmax = (arr)[i]; 
      imax = i;
    } else
      if( dmin > (arr)[i]){
	dmin = (arr)[i];
	imin = i;
      }    
    mf += (arr)[i];
  }
  mf -=dmax;
  a_mean = mf/(double)(ndat-1);
  snr = dmax/a_mean;

}

template <class T>
void Maxmin_db <T>::Max_min1( const T* arr, const long n)
{
  ndat = n;
  
  if(ndat <1 ) Warn("max_min1");
  imax = imin =0;
  dmax = dmin = arr[0];
  
  double mf=0;    
  for(long i=0; i<ndat; i++){
    if( dmax < (arr)[i]){
      dmax = (arr)[i]; 
      imax = i;
    } else
      if( dmin > (arr)[i]){
	dmin = (arr)[i];
	imin = i;
      }    
    mf += (arr)[i];
  }
  mf -=dmax;
  a_mean = mf/(double)(ndat-1);
  snr = dmax/a_mean;

}

#endif
